@@ -4,9 +4,10 @@ rocks_trees = {
    { name = "user", root = home .. "/.luarocks" };
    { name = "system", root = "/usr" };
 }
-lua_interpreter = "lua5.3";
 variables = {
    LUA_DIR = "/usr";
-   LUA_INCDIR = "/usr/include/lua5.3";
+   LUA_INCDIR = "/usr/include/lua5.1";
    LUA_BINDIR = "/usr/bin";
+   LUA_VERSION = "5.1";
+   LUA = "/usr/bin/lua5.4";
 }
