@@ -9,6 +9,7 @@ local fs = require("luarocks.fs")
 local dir = require("luarocks.dir")
 local deps = require("luarocks.deps")
 local cfg = require("luarocks.core.cfg")
+local vers = require("luarocks.core.vers")
 local repos = require("luarocks.repos")
 local writer = require("luarocks.manif.writer")
 local deplocks = require("luarocks.deplocks")
@@ -23,6 +24,7 @@ build.opts = util.opts_table("build.opts", {
    verify = "boolean",
    check_lua_versions = "boolean",
    pin = "boolean",
+   no_install = "boolean"
 })
 
 do
@@ -81,9 +83,6 @@ end
 
 local function check_macosx_deployment_target(rockspec)
    local target = rockspec.build.macosx_deployment_target
-   local function minor(version) 
-      return tonumber(version and version:match("^[^.]+%.([^.]+)"))
-   end
    local function patch_variable(var)
       if rockspec.variables[var]:match("MACOSX_DEPLOYMENT_TARGET") then
          rockspec.variables[var] = (rockspec.variables[var]):gsub("MACOSX_DEPLOYMENT_TARGET=[^ ]*", "MACOSX_DEPLOYMENT_TARGET="..target)
@@ -93,10 +92,10 @@ local function check_macosx_deployment_target(rockspec)
    end
    if cfg.is_platform("macosx") and rockspec:format_is_at_least("3.0") and target then
       local version = util.popen_read("sw_vers -productVersion")
-      local versionminor = minor(version)
-      local targetminor = minor(target)
-      if targetminor > versionminor then
-         return nil, ("This rock requires Mac OSX 10.%d, and you are running 10.%d."):format(targetminor, versionminor)
+      if version:match("^%d+%.%d+%.%d+$") or version:match("^%d+%.%d+$") then
+         if vers.compare_versions(target, version) then
+            return nil, ("This rock requires Mac OSX %s, and you are running %s."):format(target, version)
+         end
       end
       patch_variable("CC")
       patch_variable("LD")
@@ -112,35 +111,50 @@ local function process_dependencies(rockspec, opts)
       end
    end
 
-   local ok, err, errcode = deps.check_lua_incdir(rockspec.variables)
-   if not ok then
-      return nil, err, errcode
-   end
-
-   if cfg.link_lua_explicitly then
-      local ok, err, errcode = deps.check_lua_libdir(rockspec.variables)
-      if not ok then
-         return nil, err, errcode
-      end
-   end
-
    if opts.deps_mode == "none" then
       return true
    end
 
    if not opts.build_only_deps then
       if next(rockspec.build_dependencies) then
-         local ok, err, errcode = deps.fulfill_dependencies(rockspec, "build_dependencies", opts.deps_mode, opts.verify)
+
+         local user_lua_version = cfg.lua_version
+         local running_lua_version = _VERSION:sub(5)
+
+         if running_lua_version ~= user_lua_version then
+            -- Temporarily flip the user-selected Lua version,
+            -- so that we install build dependencies for the
+            -- Lua version on which the LuaRocks program is running.
+
+            -- HACK: we have to do this by flipping a bunch of
+            -- global config settings, and this list may not be complete.
+            cfg.lua_version = running_lua_version
+            cfg.lua_modules_path = cfg.lua_modules_path:gsub(user_lua_version:gsub("%.", "%%."), running_lua_version)
+            cfg.lib_modules_path = cfg.lib_modules_path:gsub(user_lua_version:gsub("%.", "%%."), running_lua_version)
+            cfg.rocks_subdir = cfg.rocks_subdir:gsub(user_lua_version:gsub("%.", "%%."), running_lua_version)
+            path.use_tree(cfg.root_dir)
+         end
+
+         local ok, err, errcode = deps.fulfill_dependencies(rockspec, "build_dependencies", "all", opts.verify)
+
+         path.add_to_package_paths(cfg.root_dir)
+
+         if running_lua_version ~= user_lua_version then
+            -- flip the settings back
+            cfg.lua_version = user_lua_version
+            cfg.lua_modules_path = cfg.lua_modules_path:gsub(running_lua_version:gsub("%.", "%%."), user_lua_version)
+            cfg.lib_modules_path = cfg.lib_modules_path:gsub(running_lua_version:gsub("%.", "%%."), user_lua_version)
+            cfg.rocks_subdir = cfg.rocks_subdir:gsub(running_lua_version:gsub("%.", "%%."), user_lua_version)
+            path.use_tree(cfg.root_dir)
+         end
+
          if err then
             return nil, err, errcode
          end
       end
    end
-   local ok, err, errcode = deps.fulfill_dependencies(rockspec, "dependencies", opts.deps_mode, opts.verify)
-   if err then
-      return nil, err, errcode
-   end
-   return true
+
+   return deps.fulfill_dependencies(rockspec, "dependencies", opts.deps_mode, opts.verify)
 end
 
 local function fetch_and_change_to_source_dir(rockspec, opts)
@@ -160,8 +174,14 @@ local function fetch_and_change_to_source_dir(rockspec, opts)
       if not ok then
          return nil, err
       end
-   elseif rockspec.source.file then
-      local ok, err = fs.unpack_archive(rockspec.source.file)
+   else
+      if rockspec.source.file then
+         local ok, err = fs.unpack_archive(rockspec.source.file)
+         if not ok then
+            return nil, err
+         end
+      end
+      local ok, err = fetch.find_rockspec_source_dir(rockspec, ".")
       if not ok then
          return nil, err
       end
@@ -188,7 +208,7 @@ local function prepare_install_dirs(name, version)
    return dirs
 end
 
-local function run_build_driver(rockspec)
+local function run_build_driver(rockspec, no_install)
    local btype = rockspec.build.type
    if btype == "none" then
       return true
@@ -206,7 +226,22 @@ local function run_build_driver(rockspec)
    if not pok or type(driver) ~= "table" then
       return nil, "Failed initializing build back-end for build type '"..btype.."': "..driver
    end
-   local ok, err = driver.run(rockspec)
+
+   if not driver.skip_lua_inc_lib_check then
+      local ok, err, errcode = deps.check_lua_incdir(rockspec.variables)
+      if not ok then
+         return nil, err, errcode
+      end
+
+      if cfg.link_lua_explicitly then
+         local ok, err, errcode = deps.check_lua_libdir(rockspec.variables)
+         if not ok then
+            return nil, err, errcode
+         end
+      end
+   end
+
+   local ok, err = driver.run(rockspec, no_install)
    if not ok then
       return nil, "Build error: " .. err
    end
@@ -261,7 +296,7 @@ do
             local ok, err = fs.make_dir(dest)
             if not ok then return nil, err end
          end
-         local ok = fs.copy(dir.path(file), dir.path(dest, filename), perms)
+         local ok = fs.copy(file, dir.path(dest, filename), perms)
          if not ok then
             return nil, "Failed copying "..file
          end
@@ -329,7 +364,7 @@ local function write_rock_dir_files(rockspec, opts)
    local name, version = rockspec.name, rockspec.version
 
    fs.copy(rockspec.local_abs_filename, path.rockspec_file(name, version), "read")
-   
+
    local deplock_file = deplocks.get_abs_filename(rockspec.name)
    if deplock_file then
       fs.copy(deplock_file, dir.path(path.install_dir(name, version), "luarocks.lock"), "read")
@@ -376,38 +411,54 @@ function build.build_rockspec(rockspec, opts)
    if opts.pin then
       deplocks.init(rockspec.name, ".")
    end
- 
+
    ok, err = process_dependencies(rockspec, opts)
    if not ok then return nil, err end
 
    local name, version = rockspec.name, rockspec.version
    if opts.build_only_deps then
+      if opts.pin then
+         deplocks.write_file()
+      end
       return name, version
-   end   
-
-   if repos.is_installed(name, version) then
-      repos.delete_version(name, version, opts.deps_mode)
    end
-   local dirs, err = prepare_install_dirs(name, version)
-   if not dirs then return nil, err end
-   
-   local rollback = util.schedule_function(function()
-      fs.delete(path.install_dir(name, version))
-      fs.remove_dir_if_empty(path.versions_dir(name))
-   end)
+
+   local dirs, err
+   local rollback
+   if not opts.no_install then
+      if repos.is_installed(name, version) then
+         repos.delete_version(name, version, opts.deps_mode)
+      end
+
+      dirs, err = prepare_install_dirs(name, version)
+      if not dirs then return nil, err end
+
+      rollback = util.schedule_function(function()
+         fs.delete(path.install_dir(name, version))
+         fs.remove_dir_if_empty(path.versions_dir(name))
+      end)
+   end
 
    ok, err = build.apply_patches(rockspec)
    if not ok then return nil, err end
-   
+
    ok, err = check_macosx_deployment_target(rockspec)
    if not ok then return nil, err end
-   
-   ok, err = run_build_driver(rockspec)
+
+   ok, err = run_build_driver(rockspec, opts.no_install)
    if not ok then return nil, err end
-   
+
+   if opts.no_install then
+      fs.pop_dir()
+      if opts.need_to_fetch then
+         fs.pop_dir()
+      end
+      return name, version
+   end
+
    ok, err = install_files(rockspec, dirs)
    if not ok then return nil, err end
-   
+
    for _, d in pairs(dirs) do
       fs.remove_dir_if_empty(d.name)
    end
@@ -426,7 +477,7 @@ function build.build_rockspec(rockspec, opts)
 
    ok, err = repos.deploy_files(name, version, repos.should_wrap_bin_scripts(rockspec), opts.deps_mode)
    if not ok then return nil, err end
-   
+
    util.remove_scheduled_function(rollback)
    rollback = util.schedule_function(function()
       repos.delete_version(name, version, opts.deps_mode)
