@@ -18,7 +18,7 @@ local make = require("luarocks.cmd.make")
 local cmd = require("luarocks.cmd")
 
 function cmd_build.add_to_parser(parser)
-   local cmd = parser:command("build", "Build and install a rock, compiling its C parts if any.\n"..
+   local cmd = parser:command("build", "Build and install a rock, compiling its C parts if any.\n"..  -- luacheck: ignore 431
       "If the sources contain a luarocks.lock file, uses it as an authoritative source for "..
       "exact version of dependencies.\n"..
       "If no arguments are given, behaves as luarocks make.", util.see_also())
@@ -36,7 +36,7 @@ function cmd_build.add_to_parser(parser)
       "rockspec. Allows to specify a different branch to fetch. Particularly "..
       'for "dev" rocks.')
       :argname("<name>")
-   parser:flag("--pin", "Create a luarocks.lock file listing the exact "..
+   cmd:flag("--pin", "Create a luarocks.lock file listing the exact "..
       "versions of each dependency found for this rock (recursively), "..
       "and store it in the rock's directory. "..
       "Ignores any existing luarocks.lock file in the rock's sources.")
@@ -94,9 +94,9 @@ local function do_build(name, namespace, version, opts)
    end
 
    if url:match("%.rockspec$") then
-      local rockspec, err, errcode = fetch.load_rockspec(url, nil, opts.verify)
+      local rockspec, err = fetch.load_rockspec(url, nil, opts.verify)
       if not rockspec then
-         return nil, err, errcode
+         return nil, err
       end
       return build.build_rockspec(rockspec, opts)
    end
@@ -138,22 +138,22 @@ function cmd_build.command(args)
 
    if args.pack_binary_rock then
       return pack.pack_binary_rock(args.rock, args.namespace, args.version, args.sign, function()
-         local name, version, errcode = do_build(args.rock, args.namespace, args.version, opts)
+         local name, version = do_build(args.rock, args.namespace, args.version, opts)
          if name and args.no_doc then
             util.remove_doc_dir(name, version)
          end
-         return name, version, errcode
+         return name, version
       end)
    end
-   
+
    local ok, err = fs.check_command_permissions(args)
    if not ok then
       return nil, err, cmd.errorcodes.PERMISSIONDENIED
    end
 
-   local name, version, errcode = do_build(args.rock, args.namespace, args.version, opts)
+   local name, version = do_build(args.rock, args.namespace, args.version, opts)
    if not name then
-      return nil, version, errcode
+      return nil, version
    end
 
    if args.no_doc then
@@ -165,8 +165,10 @@ function cmd_build.command(args)
       util.printout()
    else
       if (not args.keep) and not cfg.keep_other_versions then
-         local ok, err = remove.remove_other_versions(name, version, args.force, args.force_fast)
+         local ok, err, warn = remove.remove_other_versions(name, version, args.force, args.force_fast)
          if not ok then
+            return nil, err
+         elseif warn then
             util.printerr(err)
          end
       end
@@ -178,4 +180,6 @@ function cmd_build.command(args)
    return name, version
 end
 
+cmd_build.needs_lock = true
+
 return cmd_build
