@@ -8,6 +8,7 @@ local util = require("luarocks.util")
 local deps = require("luarocks.deps")
 local dir = require("luarocks.dir")
 local fs = require("luarocks.fs")
+local json = require("luarocks.vendor.dkjson")
 
 function config_cmd.add_to_parser(parser)
    local cmd = parser:command("config", [[
@@ -18,7 +19,6 @@ Query information about the LuaRocks configuration.
   any command-line flags passed)
 
   Examples:
-     luarocks config lua_interpreter
      luarocks config variables.LUA_INCDIR
      luarocks config lua_version
 
@@ -29,7 +29,7 @@ Query information about the LuaRocks configuration.
   * `lua_dir` is a special key as it checks for a valid Lua installation
     (equivalent to --lua-dir) and sets several keys at once.
   * `lua_version` is a special key as it changes the default Lua version
-    used by LuaRocks commands (eqivalent to passing --lua-version). 
+    used by LuaRocks commands (equivalent to passing --lua-version).
 
   Examples:
      luarocks config variables.OPENSSL_DIR /usr/local/openssl
@@ -114,7 +114,7 @@ local function traverse_varstring(var, tbl, fn, missing_parent)
          k, r = var:match("^([^[]+)(%[.*)$")
       end
    end
-   
+
    if k then
       if not tbl[k] and missing_parent then
          missing_parent(tbl, k)
@@ -131,16 +131,11 @@ local function traverse_varstring(var, tbl, fn, missing_parent)
    if i then
       var = tonumber(i)
    end
-   
+
    return fn(tbl, var)
 end
 
 local function print_json(value)
-   local json_ok, json = util.require_json()
-   if not json_ok then
-      return nil, "A JSON library is required for this command. "..json
-   end
-
    print(json.encode(value))
    return true
 end
@@ -179,12 +174,14 @@ local function write_entries(keys, scope, do_unset)
    if scope == "project" and not cfg.config_files.project then
       return nil, "Current directory is not part of a project. You may want to run `luarocks init`."
    end
-   
-   local tbl, err = persist.load_config_file_if_basic(cfg.config_files[scope].file, cfg)
+
+   local file_name = cfg.config_files[scope].file
+
+   local tbl, err = persist.load_config_file_if_basic(file_name, cfg)
    if not tbl then
       return nil, err
    end
-   
+
    for var, val in util.sortedpairs(keys) do
       traverse_varstring(var, tbl, function(t, k)
          if do_unset then
@@ -210,18 +207,27 @@ local function write_entries(keys, scope, do_unset)
       end)
    end
 
-   local ok, err = persist.save_from_table(cfg.config_files[scope].file, tbl)
+   local ok, err = fs.make_dir(dir.dir_name(file_name))
+   if not ok then
+      return nil, err
+   end
+
+   ok, err = persist.save_from_table(file_name, tbl)
    if ok then
       print(do_unset and "Removed" or "Wrote")
       for var, val in util.sortedpairs(keys) do
          if do_unset then
             print(("\t%s"):format(var))
          else
-            print(("\t%s = %q"):format(var, val))
+            if type(val) == "string" then
+               print(("\t%s = %q"):format(var, val))
+            else
+               print(("\t%s = %s"):format(var, tostring(val)))
+            end
          end
       end
       print(do_unset and "from" or "to")
-      print("\t" .. cfg.config_files[scope].file)
+      print("\t" .. file_name)
       return true
    else
       return nil, err
@@ -233,7 +239,7 @@ local function get_scope(args)
           or (args["local"] and "user")
           or (args.project_tree and "project")
           or (cfg.local_by_default and "user")
-          or (fs.is_writable(cfg.config_files["system"].file and "system"))
+          or (fs.is_writable(cfg.config_files["system"].file) and "system")
           or "user"
 end
 
@@ -242,7 +248,7 @@ end
 function config_cmd.command(args)
    deps.check_lua_incdir(cfg.variables, args.lua_version or cfg.lua_version)
    deps.check_lua_libdir(cfg.variables, args.lua_version or cfg.lua_version)
-   
+
    -- deprecated flags
    if args.lua_incdir then
       print(cfg.variables.LUA_INCDIR)
@@ -280,14 +286,19 @@ function config_cmd.command(args)
          return nil, "Current directory is not part of a project. You may want to run `luarocks init`."
       end
 
-      local prefix = dir.dir_name(cfg.config_files[scope].file)
+      local location = cfg.config_files[scope]
+      if (not location) or (not location.file) then
+         return nil, "could not get config file location for " .. tostring(scope) .. " scope"
+      end
+
+      local prefix = dir.dir_name(location.file)
       local ok, err = persist.save_default_lua_version(prefix, args.value)
       if not ok then
          return nil, "could not set default Lua version: " .. err
       end
       print("Lua version will default to " .. args.value .. " in " .. prefix)
    end
-   
+
    if args.key == "lua_dir" and args.value then
       local scope = get_scope(args)
       local keys = {
@@ -295,7 +306,7 @@ function config_cmd.command(args)
          ["variables.LUA_BINDIR"] = cfg.variables.LUA_BINDIR,
          ["variables.LUA_INCDIR"] = cfg.variables.LUA_INCDIR,
          ["variables.LUA_LIBDIR"] = cfg.variables.LUA_LIBDIR,
-         ["lua_interpreter"] = cfg.lua_interpreter,
+         ["variables.LUA"] = cfg.variables.LUA,
       }
       if args.lua_version then
          local prefix = dir.dir_name(cfg.config_files[scope].file)
