@@ -14,10 +14,9 @@ local search = require("luarocks.search")
 local queries = require("luarocks.queries")
 local cfg = require("luarocks.core.cfg")
 local cmd = require("luarocks.cmd")
-local dir = require("luarocks.dir")
 
 function install.add_to_parser(parser)
-   local cmd = parser:command("install", "Install a rock.", util.see_also())
+   local cmd = parser:command("install", "Install a rock.", util.see_also())  -- luacheck: ignore 431
 
    cmd:argument("rock", "The name of a rock to be fetched from a repository "..
       "or a filename of a locally available rock.")
@@ -44,6 +43,10 @@ function install.add_to_parser(parser)
       "and report if it is available for another Lua version.")
    util.deps_mode_option(cmd)
    cmd:flag("--no-manifest", "Skip creating/updating the manifest")
+   cmd:flag("--pin", "If the installed rock is a Lua module, create a "..
+      "luarocks.lock file listing the exact versions of each dependency found for "..
+      "this rock (recursively), and store it in the rock's directory. "..
+      "Ignores any existing luarocks.lock file in the rock's sources.")
    -- luarocks build options
    parser:flag("--pack-binary-rock"):hidden(true)
    parser:option("--branch"):hidden(true)
@@ -76,16 +79,16 @@ function install.install_binary_rock(rock_file, opts)
    if not name then
       return nil, "Filename "..rock_file.." does not match format 'name-version-revision.arch.rock'."
    end
-   
+
    if arch ~= "all" and arch ~= cfg.arch then
       return nil, "Incompatible architecture "..arch, "arch"
    end
    if repos.is_installed(name, version) then
       repos.delete_version(name, version, opts.deps_mode)
    end
-   
+
    local install_dir = path.install_dir(name, version)
-   
+
    local rollback = util.schedule_function(function()
       fs.delete(install_dir)
       fs.remove_dir_if_empty(path.versions_dir(name))
@@ -140,7 +143,7 @@ end
 -- @param rock_file string: local or remote filename of a rock.
 -- @param opts table: installation options
 -- @return (string, string) or (nil, string, [string]): Name and version of
--- the rock whose dependencies were installed if succeeded or nil and an error message 
+-- the rock whose dependencies were installed if succeeded or nil and an error message
 -- followed by an error code.
 function install.install_binary_rock_deps(rock_file, opts)
    assert(type(rock_file) == "string")
@@ -150,7 +153,7 @@ function install.install_binary_rock_deps(rock_file, opts)
    if not name then
       return nil, "Filename "..rock_file.." does not match format 'name-version-revision.arch.rock'."
    end
-   
+
    if arch ~= "all" and arch ~= cfg.arch then
       return nil, "Incompatible architecture "..arch, "arch"
    end
@@ -159,7 +162,7 @@ function install.install_binary_rock_deps(rock_file, opts)
 
    local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, install_dir, opts.verify)
    if not ok then return nil, err, errcode end
-   
+
    local rockspec, err = fetch.load_rockspec(path.rockspec_file(name, version))
    if err then
       return nil, "Failed loading rockspec for installed package: "..err, errcode
@@ -196,8 +199,12 @@ local function install_rock_file(filename, opts)
    end
 
    if (not opts.keep) and not cfg.keep_other_versions then
-      local ok, err = remove.remove_other_versions(name, version, opts.force, opts.force_fast)
-      if not ok then util.printerr(err) end
+      local ok, err, warn = remove.remove_other_versions(name, version, opts.force, opts.force_fast)
+      if not ok then
+         return nil, err
+      elseif warn then
+         util.printerr(err)
+      end
    end
 
    writer.check_dependencies(nil, opts.deps_mode)
@@ -248,4 +255,6 @@ function install.command(args)
    end
 end
 
+install.needs_lock = true
+
 return install
