@@ -33,21 +33,28 @@ function fetch.fetch_caching(url, mirroring)
    local name = repo_url:gsub("[/:]","_")
    local cache_dir = dir.path(cfg.local_cache, name)
    local ok = fs.make_dir(cache_dir)
-   if not ok then
+   local lock = ok and fs.lock_access(cache_dir)
+   if not (ok and lock) then
       cfg.local_cache = fs.make_temp_dir("local_cache")
+      if not cfg.local_cache then
+         return nil, "Failed creating temporary local_cache directory"
+      end
       cache_dir = dir.path(cfg.local_cache, name)
       ok = fs.make_dir(cache_dir)
       if not ok then
          return nil, "Failed creating temporary cache directory "..cache_dir
       end
+      lock = fs.lock_access(cache_dir)
    end
 
    local cachefile = dir.path(cache_dir, filename)
    if cfg.aggressive_cache and (not name:match("^manifest")) and fs.exists(cachefile) then
+      fs.unlock_access(lock)
       return cachefile, nil, nil, true
    end
 
    local file, err, errcode, from_cache = fetch.fetch_url(url, cachefile, true, mirroring)
+   fs.unlock_access(lock)
    if not file then
       return nil, err or "Failed downloading "..url, errcode
    end
@@ -94,7 +101,7 @@ local function download_with_mirrors(url, filename, cache, servers)
       end
    end
 
-   return nil, err
+   return nil, err, "network"
 end
 
 --- Fetch a local or remote file.
@@ -126,7 +133,7 @@ function fetch.fetch_url(url, filename, cache, mirroring)
 
    local protocol, pathname = dir.split_url(url)
    if protocol == "file" then
-      local fullname = dir.normalize(fs.absolute_name(pathname))
+      local fullname = fs.absolute_name(pathname)
       if not fs.exists(fullname) then
          local hint = (not pathname:match("^/"))
                       and (" - note that given path in rockspec is not absolute: " .. url)
@@ -134,7 +141,7 @@ function fetch.fetch_url(url, filename, cache, mirroring)
          return nil, "Local file not found: " .. fullname .. hint
       end
       filename = filename or dir.base_name(pathname)
-      local dstname = dir.normalize(fs.absolute_name(dir.path(".", filename)))
+      local dstname = fs.absolute_name(dir.path(".", filename))
       local ok, err
       if fullname == dstname then
          ok = true
@@ -154,7 +161,7 @@ function fetch.fetch_url(url, filename, cache, mirroring)
          ok, name, from_cache = fs.download(url, filename, cache)
       end
       if not ok then
-         return nil, "Failed downloading "..url..(name and " - "..name or ""), "network"
+         return nil, "Failed downloading "..url..(name and " - "..name or ""), from_cache
       end
       return name, nil, nil, from_cache
    else
@@ -205,7 +212,9 @@ function fetch.fetch_url_at_temp_dir(url, tmpname, filename, cache)
             file = dir.path(temp_dir, filename)
             fs.copy(cachefile, file)
          end
-      else
+      end
+
+      if not file then
          file, err, errcode = fetch.fetch_url(url, filename, cache)
       end
 
@@ -234,6 +243,21 @@ function fetch.find_base_dir(file, temp_dir, src_url, src_dir)
    local ok, err = fs.change_dir(temp_dir)
    if not ok then return nil, err end
    fs.unpack_archive(file)
+
+   if not src_dir then
+      local rockspec = {
+         source = {
+            file = file,
+            dir = src_dir,
+            url = src_url,
+         }
+      }
+      ok, err = fetch.find_rockspec_source_dir(rockspec, ".")
+      if ok then
+         src_dir = rockspec.source.dir
+      end
+   end
+
    local inferred_dir = src_dir or dir.deduce_base_dir(src_url)
    local found_dir = nil
    if fs.exists(inferred_dir) then
@@ -459,35 +483,60 @@ function fetch.get_sources(rockspec, extract, dest_dir)
       if not ok then return nil, err end
       ok, err = fs.unpack_archive(rockspec.source.file)
       if not ok then return nil, err end
-      if not fs.exists(rockspec.source.dir) then
-
-         -- If rockspec.source.dir can't be found, see if we only have one
-         -- directory in store_dir.  If that's the case, assume it's what
-         -- we're looking for.
-         -- We only do this if the rockspec source.dir was not set, and only
-         -- with rockspecs newer than 3.0.
-         local file_count, found_dir = 0
-
-         if not rockspec.source.dir_set and rockspec:format_is_at_least("3.0") then
-            for file in fs.dir() do
-               file_count = file_count + 1
-               if fs.is_dir(file) then
-                  found_dir = file
-               end
-            end
-         end
-
-         if file_count == 1 and found_dir then
-            rockspec.source.dir = found_dir
-         else
-            return nil, "Directory "..rockspec.source.dir.." not found inside archive "..rockspec.source.file, "source.dir", source_file, store_dir
-         end
-      end
+      ok, err = fetch.find_rockspec_source_dir(rockspec, ".")
+      if not ok then return nil, err end
       fs.pop_dir()
    end
    return source_file, store_dir
 end
 
+function fetch.find_rockspec_source_dir(rockspec, store_dir)
+   local ok, err = fs.change_dir(store_dir)
+   if not ok then return nil, err end
+
+   local file_count, dir_count, found_dir = 0, 0, 0
+
+   if rockspec.source.dir and fs.exists(rockspec.source.dir) then
+      ok, err = true, nil
+   elseif rockspec.source.file and rockspec.source.dir then
+      ok, err = nil, "Directory "..rockspec.source.dir.." not found inside archive "..rockspec.source.file
+   elseif not rockspec.source.dir_set then -- and rockspec:format_is_at_least("3.0") then
+
+      local name = dir.base_name(rockspec.source.file or rockspec.source.url or "")
+
+      if name:match("%.lua$") or name:match("%.c$") then
+         if fs.is_file(name) then
+            rockspec.source.dir = "."
+            ok, err = true, nil
+         end
+      end
+
+      if not rockspec.source.dir then
+         for file in fs.dir() do
+            file_count = file_count + 1
+            if fs.is_dir(file) then
+               dir_count = dir_count + 1
+               found_dir = file
+            end
+         end
+
+         if dir_count == 1 then
+            rockspec.source.dir = found_dir
+            ok, err = true, nil
+         else
+            ok, err = nil, "Could not determine source directory from rock contents (" .. tostring(file_count).." file(s), "..tostring(dir_count).." dir(s))"
+         end
+      end
+   else
+      ok, err = nil, "Could not determine source directory, please set source.dir in rockspec."
+   end
+
+   fs.pop_dir()
+
+   assert(rockspec.source.dir or not ok)
+   return ok, err
+end
+
 --- Download sources for building a rock, calling the appropriate protocol method.
 -- @param rockspec table: The rockspec table
 -- @param extract boolean: When downloading compressed formats, whether to extract
@@ -511,7 +560,7 @@ function fetch.fetch_sources(rockspec, extract, dest_dir)
    end
 
    local protocol = rockspec.source.protocol
-   local ok, proto
+   local ok, err, proto
    if dir.is_basic_protocol(protocol) then
       proto = fetch
    else
@@ -531,7 +580,13 @@ function fetch.fetch_sources(rockspec, extract, dest_dir)
       end
    end
 
-   return proto.get_sources(rockspec, extract, dest_dir)
+   local source_file, store_dir = proto.get_sources(rockspec, extract, dest_dir)
+   if not source_file then return nil, store_dir end
+
+   ok, err = fetch.find_rockspec_source_dir(rockspec, store_dir)
+   if not ok then return nil, err, "source.dir", source_file, store_dir end
+
+   return source_file, store_dir
 end
 
 return fetch
