@@ -28,6 +28,8 @@ end
 local patch = require("luarocks.tools.patch")
 local tar = require("luarocks.tools.tar")
 
+local dir_sep = package.config:sub(1, 1)
+
 local dir_stack = {}
 
 --- Test is file/dir is writable.
@@ -54,7 +56,7 @@ function fs_lua.is_writable(file)
    return result
 end
 
-local function quote_args(command, ...)
+function fs_lua.quote_args(command, ...)
    local out = { command }
    local args = pack(...)
    for i=1, args.n do
@@ -74,7 +76,7 @@ end
 -- otherwise.
 function fs_lua.execute(command, ...)
    assert(type(command) == "string")
-   return fs.execute_string(quote_args(command, ...))
+   return fs.execute_string(fs.quote_args(command, ...))
 end
 
 --- Run the given command, quoting its arguments, silencing its output.
@@ -88,19 +90,19 @@ end
 function fs_lua.execute_quiet(command, ...)
    assert(type(command) == "string")
    if cfg.verbose then -- omit silencing output
-      return fs.execute_string(quote_args(command, ...))
+      return fs.execute_string(fs.quote_args(command, ...))
    else
-      return fs.execute_string(fs.quiet(quote_args(command, ...)))
+      return fs.execute_string(fs.quiet(fs.quote_args(command, ...)))
    end
 end
 
-function fs.execute_env(env, command, ...)
+function fs_lua.execute_env(env, command, ...)
    assert(type(command) == "string")
    local envstr = {}
    for var, val in pairs(env) do
       table.insert(envstr, fs.export_cmd(var, val))
    end
-   return fs.execute_string(table.concat(envstr, "\n") .. "\n" .. quote_args(command, ...))
+   return fs.execute_string(table.concat(envstr, "\n") .. "\n" .. fs.quote_args(command, ...))
 end
 
 local tool_available_cache = {}
@@ -114,19 +116,30 @@ end
 -- The tool is executed using a flag, usually just to ask its version.
 -- @param tool_cmd string: The command to be used to check the tool's presence (e.g. hg in case of Mercurial)
 -- @param tool_name string: The actual name of the tool (e.g. Mercurial)
--- @param arg string: The flag to pass to the tool. '--version' by default.
-function fs_lua.is_tool_available(tool_cmd, tool_name, arg)
+function fs_lua.is_tool_available(tool_cmd, tool_name)
    assert(type(tool_cmd) == "string")
    assert(type(tool_name) == "string")
 
-   arg = arg or "--version"
-   assert(type(arg) == "string")
-
    local ok
    if tool_available_cache[tool_name] ~= nil then
       ok = tool_available_cache[tool_name]
    else
-      ok = fs.execute_quiet(tool_cmd, arg)
+      local tool_cmd_no_args = tool_cmd:gsub(" [^\"]*$", "")
+
+      -- if it looks like the tool has a pathname, try that first
+      if tool_cmd_no_args:match("[/\\]") then
+         local tool_cmd_no_args_normalized = dir.path(tool_cmd_no_args)
+         local fd = io.open(tool_cmd_no_args_normalized, "r")
+         if fd then
+            fd:close()
+            ok = true
+         end
+      end
+
+      if not ok then
+         ok = fs.search_in_path(tool_cmd_no_args)
+      end
+
       tool_available_cache[tool_name] = (ok == true)
    end
 
@@ -266,12 +279,29 @@ function fs_lua.system_temp_dir()
    return os.getenv("TMPDIR") or os.getenv("TEMP") or "/tmp"
 end
 
+local function temp_dir_pattern(name_pattern)
+   return dir.path(fs.system_temp_dir(),
+                   "luarocks_" .. dir.normalize(name_pattern):gsub("[/\\]", "_") .. "-")
+end
+
 ---------------------------------------------------------------------
 -- LuaFileSystem functions
 ---------------------------------------------------------------------
 
 if lfs_ok then
 
+function fs_lua.lock_access(dirname, force)
+   fs.make_dir(dirname)
+   if force then
+      os.remove(dir.path(dirname, "lockfile.lfs"))
+   end
+   return lfs.lock_dir(dirname)
+end
+
+function fs_lua.unlock_access(lock)
+   return lock:free()
+end
+
 --- Run the given command.
 -- The command is executed in the current directory in the dir stack.
 -- @param cmd string: No quoting/escaping is applied to the command.
@@ -338,21 +368,23 @@ function fs_lua.make_dir(directory)
      path = directory:sub(1, 2)
      directory = directory:sub(4)
    else
-     if directory:match("^/") then
+     if directory:match("^" .. dir_sep) then
         path = ""
      end
    end
-   for d in directory:gmatch("([^/]+)/*") do
-      path = path and path .. "/" .. d or d
+   for d in directory:gmatch("([^" .. dir_sep .. "]+)" .. dir_sep .. "*") do
+      path = path and path .. dir_sep .. d or d
       local mode = lfs.attributes(path, "mode")
       if not mode then
          local ok, err = lfs.mkdir(path)
          if not ok then
             return false, err
          end
-         ok, err = fs.set_permissions(path, "exec", "all")
-         if not ok then
-            return false, err
+         if cfg.is_platform("unix") then
+            ok, err = fs.set_permissions(path, "exec", "all")
+            if not ok then
+               return false, err
+            end
          end
       elseif mode ~= "directory" then
          return false, path.." is not a directory"
@@ -479,7 +511,8 @@ end
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
 function fs_lua.copy_contents(src, dest, perms)
-   assert(src and dest)
+   assert(src)
+   assert(dest)
    src = dir.normalize(src)
    dest = dir.normalize(dest)
    if not fs.is_dir(src) then
@@ -564,7 +597,7 @@ local function recursive_find(cwd, prefix, result)
          table.insert(result, item)
          local pathname = dir.path(cwd, file)
          if lfs.attributes(pathname, "mode") == "directory" then
-            recursive_find(pathname, item.."/", result)
+            recursive_find(pathname, item .. dir_sep, result)
          end
       end
    end
@@ -631,9 +664,8 @@ else -- if not lfs_ok
 
 function fs_lua.exists(file)
    assert(file)
-   file = dir.normalize(fs.absolute_name(file))
    -- check if file exists by attempting to open it
-   return util.exists(file)
+   return util.exists(fs.absolute_name(file))
 end
 
 end
@@ -919,7 +951,7 @@ function fs_lua.download(url, filename, cache)
       end
       return fs.use_downloader(url, filename, cache)
    elseif not ok then
-      return nil, err
+      return nil, err, "network"
    end
    return true, filename, from_cache
 end
@@ -1035,7 +1067,7 @@ function fs_lua.current_user()
 end
 
 function fs_lua.is_superuser()
-   return false
+   return posix.geteuid() == 0
 end
 
 -- This call is not available on all systems, see #677
@@ -1047,9 +1079,8 @@ if posix.mkdtemp then
 -- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.
 function fs_lua.make_temp_dir(name_pattern)
    assert(type(name_pattern) == "string")
-   name_pattern = dir.normalize(name_pattern)
 
-   return posix.mkdtemp(fs.system_temp_dir() .. "/luarocks_" .. name_pattern:gsub("/", "_") .. "-XXXXXX")
+   return posix.mkdtemp(temp_dir_pattern(name_pattern) .. "-XXXXXX")
 end
 
 end -- if posix.mkdtemp
@@ -1060,20 +1091,21 @@ end
 -- Other functions
 ---------------------------------------------------------------------
 
-if lfs_ok and not fs_lua.make_temp_dir then
+if not fs_lua.make_temp_dir then
 
 function fs_lua.make_temp_dir(name_pattern)
    assert(type(name_pattern) == "string")
-   name_pattern = dir.normalize(name_pattern)
 
-   local pattern = fs.system_temp_dir() .. "/luarocks_" .. name_pattern:gsub("/", "_") .. "-"
-
-   while true do
-      local name = pattern .. tostring(math.random(10000000))
-      if lfs.mkdir(name) then
+   local ok, err
+   for _ = 1, 3 do
+      local name = temp_dir_pattern(name_pattern) .. tostring(math.random(10000000))
+      ok, err = fs.make_dir(name)
+      if ok then
          return name
       end
    end
+
+   return nil, err
 end
 
 end
@@ -1165,7 +1197,7 @@ end
 -- @return boolean true, if it is a Lua script, false otherwise
 function fs_lua.is_lua(filename)
   filename = filename:gsub([[%\]],"/")   -- normalize on fw slash to prevent escaping issues
-  local lua = fs.Q(dir.path(cfg.variables["LUA_BINDIR"], cfg.lua_interpreter))  -- get lua interpreter configured
+  local lua = fs.Q(cfg.variables.LUA)  -- get lua interpreter configured
   -- execute on configured interpreter, might not be the same as the interpreter LR is run on
   local result = fs.execute_string(lua..[[ -e "if loadfile(']]..filename..[[') then os.exit(0) else os.exit(1) end"]])
   return (result == true)
