@@ -47,11 +47,11 @@ function unix.absolute_name(pathname, relative_to)
       pathname = unquoted
    end
 
-   relative_to = (relative_to or fs.current_dir()):gsub("/*$", "")
+   relative_to = relative_to or fs.current_dir()
    if pathname:sub(1,1) == "/" then
-      return pathname
+      return dir.normalize(pathname)
    else
-      return relative_to .. "/" .. pathname
+      return dir.path(relative_to, pathname)
    end
 end
 
@@ -76,7 +76,7 @@ function unix.wrap_script(script, target, deps_mode, name, version, ...)
    assert(type(deps_mode) == "string")
    assert(type(name) == "string" or not name)
    assert(type(version) == "string" or not version)
-   
+
    local wrapper = io.open(target, "w")
    if not wrapper then
       return nil, "Could not open "..target.." for writing."
@@ -90,7 +90,8 @@ function unix.wrap_script(script, target, deps_mode, name, version, ...)
    }
 
    local remove_interpreter = false
-   if target == "luarocks" or target == "luarocks-admin" then
+   local base = dir.base_name(target):gsub("%..*$", "")
+   if base == "luarocks" or base == "luarocks-admin" then
       if cfg.is_binary then
          remove_interpreter = true
       end
@@ -107,7 +108,7 @@ function unix.wrap_script(script, target, deps_mode, name, version, ...)
    end
 
    local argv = {
-      fs.Q(dir.path(cfg.variables["LUA_BINDIR"], cfg.lua_interpreter)),
+      fs.Q(cfg.variables["LUA"]),
       "-e",
       fs.Q(table.concat(luainit, ";")),
       script and fs.Q(script) or [[$([ "$*" ] || echo -i)]],
@@ -153,7 +154,7 @@ function unix.is_actual_binary(filename)
    return first ~= "#!"
 end
 
-function unix.copy_binary(filename, dest) 
+function unix.copy_binary(filename, dest)
    return fs.copy(filename, dest, "exec")
 end
 
@@ -175,10 +176,6 @@ function unix.tmpname()
    return os.tmpname()
 end
 
-function unix.current_user()
-   return os.getenv("USER")
-end
-
 function unix.export_cmd(var, val)
    return ("export %s='%s'"):format(var, val)
 end
@@ -229,4 +226,25 @@ function unix.system_cache_dir()
    return dir.path(fs.system_temp_dir(), "cache")
 end
 
+function unix.search_in_path(program)
+   if program:match("/") then
+      local fd = io.open(dir.path(program), "r")
+      if fd then
+         fd:close()
+         return true, program
+      end
+
+      return false
+   end
+
+   for d in (os.getenv("PATH") or ""):gmatch("([^:]+)") do
+      local fd = io.open(dir.path(d, program), "r")
+      if fd then
+         fd:close()
+         return true, d
+      end
+   end
+   return false
+end
+
 return unix
