@@ -51,7 +51,7 @@ end
 --- Copy a file.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
--- @param perm string ("read" or "exec") or nil: Permissions for destination 
+-- @param perm string ("read" or "exec") or nil: Permissions for destination
 -- file or nil to use the source permissions
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
@@ -81,7 +81,10 @@ end
 -- plus an error message.
 function tools.copy_contents(src, dest)
    assert(src and dest)
-   if fs.execute_quiet(vars.CP.." -pPR "..fs.Q(src).."/* "..fs.Q(dest)) then
+   if not fs.is_dir(src) then
+      return false, src .. " is not a directory"
+   end
+   if fs.make_dir(dest) and fs.execute_quiet(vars.CP.." -pPR "..fs.Q(src).."/* "..fs.Q(dest)) then
       return true
    else
       return false, "Failed copying "..src.." to "..dest
@@ -141,7 +144,7 @@ end
 -- @return boolean: true on success, nil and error message on failure.
 function tools.unzip(zipfile)
    assert(zipfile)
-   local ok, err = fs.is_tool_available(vars.UNZIP, "unzip", "--help")
+   local ok, err = fs.is_tool_available(vars.UNZIP, "unzip")
    if not ok then
       return nil, err
    end
@@ -297,4 +300,58 @@ function tools.is_file(file)
    return fs.execute(vars.TEST, "-f", file)
 end
 
+function tools.current_user()
+   local user = os.getenv("USER")
+   if user then
+      return user
+   end
+   local pd = io.popen("whoami", "r")
+   if not pd then
+      return ""
+   end
+   user = pd:read("*l")
+   pd:close()
+   return user
+end
+
+function tools.is_superuser()
+   return fs.current_user() == "root"
+end
+
+function tools.lock_access(dirname, force)
+   fs.make_dir(dirname)
+
+   local tempfile = dir.path(dirname, ".lock.tmp." .. tostring(math.random(100000000)))
+
+   local fd, fderr = io.open(tempfile, "w")
+   if not fd then
+      return nil, "failed opening temp file " .. tempfile .. " for locking: " .. fderr
+   end
+
+   local ok, werr = fd:write("lock file for " .. dirname)
+   if not ok then
+      return nil, "failed writing temp file " .. tempfile .. " for locking: " .. werr
+   end
+
+   fd:close()
+
+   local lockfile = dir.path(dirname, "lockfile.luarocks")
+
+   local force_flag = force and " -f" or ""
+
+   if fs.execute(vars.LN .. force_flag, tempfile, lockfile) then
+      return {
+         tempfile = tempfile,
+         lockfile = lockfile,
+      }
+   else
+      return nil, "File exists" -- same message as luafilesystem
+   end
+end
+
+function tools.unlock_access(lock)
+   os.remove(lock.lockfile)
+   os.remove(lock.tempfile)
+end
+
 return tools
