@@ -30,7 +30,7 @@ local function store_package_items(storage, name, version, items)
 
    local package_identifier = name.."/"..version
 
-   for item_name, path in pairs(items) do
+   for item_name, path in pairs(items) do  -- luacheck: ignore 431
       if not storage[item_name] then
          storage[item_name] = {}
       end
@@ -54,18 +54,27 @@ local function remove_package_items(storage, name, version, items)
 
    local package_identifier = name.."/"..version
 
-   for item_name, path in pairs(items) do
-      local all_identifiers = storage[item_name]
-
-      for i, identifier in ipairs(all_identifiers) do
-         if identifier == package_identifier then
-            table.remove(all_identifiers, i)
-            break
-         end
+   for item_name, path in pairs(items) do  -- luacheck: ignore 431
+      local key = item_name
+      local all_identifiers = storage[key]
+      if not all_identifiers then
+         key = key .. ".init"
+         all_identifiers = storage[key]
       end
 
-      if #all_identifiers == 0 then
-         storage[item_name] = nil
+      if all_identifiers then
+         for i, identifier in ipairs(all_identifiers) do
+            if identifier == package_identifier then
+               table.remove(all_identifiers, i)
+               break
+            end
+         end
+
+         if #all_identifiers == 0 then
+            storage[key] = nil
+         end
+      else
+         util.warning("Cannot find entry for " .. item_name .. " in manifest -- corrupted manifest?")
       end
    end
 end
@@ -81,14 +90,18 @@ end
 local function update_dependencies(manifest, deps_mode)
    assert(type(manifest) == "table")
    assert(type(deps_mode) == "string")
-   
+
+   if not manifest.dependencies then manifest.dependencies = {} end
+   local mdeps = manifest.dependencies
+
    for pkg, versions in pairs(manifest.repository) do
       for version, repositories in pairs(versions) do
          for _, repo in ipairs(repositories) do
             if repo.arch == "installed" then
-               repo.dependencies = {}
-               deps.scan_deps(repo.dependencies, manifest, pkg, version, deps_mode)
-               repo.dependencies[pkg] = nil
+               local rd = {}
+               repo.dependencies = rd
+               deps.scan_deps(rd, mdeps, pkg, version, deps_mode)
+               rd[pkg] = nil
             end
          end
       end
@@ -151,7 +164,7 @@ local function filter_by_lua_version(manifest, lua_version, repodir, cache)
    assert(type(manifest) == "table")
    assert(type(repodir) == "string")
    assert((not cache) or type(cache) == "table")
-   
+
    cache = cache or {}
    lua_version = vers.parse_version(lua_version)
    for pkg, versions in pairs(manifest.repository) do
@@ -167,7 +180,7 @@ local function filter_by_lua_version(manifest, lua_version, repodir, cache)
                if rockspec then
                   cache[pathname] = rockspec
                   for _, dep in ipairs(rockspec.dependencies) do
-                     if dep.name == "lua" then 
+                     if dep.name == "lua" then
                         if not vers.match_constraints(lua_version, dep.constraints) then
                            table.insert(to_remove, version)
                         end
